@@ -1,12 +1,12 @@
-from .get_answer import get_answer_online
 from .get_answer import ModelManager
 from .get_answer import load_api_keys
 from .get_answer import get_answer
+from .get_answer import get_available_models
 
 
 __all__ = [
-    "get_answer_online",
     "ModelManager",
     "load_api_keys",
     "get_answer",
+    "get_available_models",
 ]
\ No newline at end of file
