@@ -1,10 +1,12 @@
 from .stat import chi_squared
+from .stat import reduced_chi_squared
 from .stat import mean_squared_error
 from .integral import integral_1d_func
 
 
 __all__ = [
     "chi_squared",
+    "reduced_chi_squared",
     "mean_squared_error",
     "integral_1d_func",
 ]
\ No newline at end of file
