@@ -1,8 +1,10 @@
 from .task_runner import execute_command
 from .task_runner import execute_python_script
+from .task_runner import run_tasks_concurrently
 
 
 __all__ = [
     "execute_command",
     "execute_python_script",
+    "run_tasks_concurrently",
 ]
\ No newline at end of file
