@@ -3,10 +3,12 @@ from typing import Set
 from typing import Dict
 from typing import List
 from typing import Tuple
+from typing import Union
 from typing import TypeVar
 from typing import Literal
 from typing import Callable
 from typing import Optional
+from typing import Coroutine
 
 
 __all__ = [
@@ -15,8 +17,10 @@ __all__ = [
     "Dict",
     "List",
     "Tuple",
+    "Union",
     "TypeVar",
     "Literal",
     "Callable",
     "Optional",
+    "Coroutine",
 ]
\ No newline at end of file
